import xml.etree.ElementTree as ET
import datetime, requests, re

StopColumns = {
    'SDURAKKODU'     : 'StopCode',
    'SDURAKADI'      : 'StopName',
    'KOORDINAT'      : 'KOORDINAT',
    'ILCEADI'        : 'District',
    'SYON'           : 'Direction',
    'AKILLI'         : 'IsSmart',
    'FIZIKI'         : 'PhysicalCondition',
    'DURAK_TIPI'     : 'Type',
    'ENGELLIKULLANIM': 'Accessibility'
    }

GarageColumns = {
    'ID'        : 'GarageId',
    'GARAJ_ADI' : 'GarageName',
    'GARAJ_KODU': 'GarageCode',
    'KOORDINAT' : 'KOORDINAT'
    }

AnnouncementColumns = {
    'HATKODU'         : 'LineCode',
    'HAT'             : 'LineName',
    'TIP'             : 'Type',
    'GUNCELLEME_SAATI': 'UpdateTime',
    'MESAJ'           : 'Message'
    }

BadRoadColums = {
    'NMESAJID'     : 'MessageId',
    'SKAPINUMARASI': 'BusId',
    'SSOFORSICILNO': 'DriveId',
    'SMESAJMETNI'  : 'Message',
    'DTKAYITZAMANI': 'UpdateDate',
    'NBOYLAM'      : 'Longitude',
    'NENLEM'       : 'Latitude'
    }

AccidentColums = {
    'DTOLAYBASLANGICZAMANI': 'AccidentDate',
    'NBOYLAM'              : 'Longitude',
    'NENLEM'               : 'Latitude',
    'Tur'                  : 'Type',
    }

LineStopColums = {
    'HATKODU'        : 'LineCode',
    'YON'            : 'Direction',
    'SIRANO'         : 'SequenceNo',
    'DURAKKODU'      : 'StopCode',
    'DURAKADI'       : 'StopName',
    'XKOORDINATI'    : 'Longitude',
    'YKOORDINATI'    : 'Latitude',
    'DURAKTIPI'      : 'Type',
    'ISLETMEBOLGE'   : 'BusinessRegion',
    'ISLETMEALTBOLGE': 'SubRegion',
    'ILCEADI'        : 'District'
    }

LineDetailColumns = {
    'HAT_KODU'    : 'LineCode',
    'HAT_ADI'     : 'LineName',
    'TAM_HAT_ADI' : 'LineFullName',
    'HAT_DURUMU'  : 'LineStatus',
    'BOLGE'       : 'BusinessRegion',
    'SEFER_SURESI': 'TravelTime'
    }

def GetStopByCode(stopCode):
    url = "https://api.ibb.gov.tr/iett/UlasimAnaVeri/HatDurakGuzergah.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/GetDurak_XML",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = f"""<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                   <Body>
                       <GetDurak_XML xmlns="http://tempuri.org/">
                           <DurakKodu>{stopCode}</DurakKodu>
                       </GetDurak_XML>
                   </Body>
               </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()

    root = ET.fromstring(r.content)
    if table := root.findall('*//Table'):
        table = {StopColumns[it.tag]: it.text for it in table[0].iter() if it.text}
        if coords := table.pop('KOORDINAT', None):
            pattern = r'POINT\s\((?P<Longitude>.*)\s(?P<Latitude>.*)\)'
            if matches := re.match(pattern, coords):
                table.update({key: float(value) for key, value in matches.groupdict().items()})
        return table
    return None

def GetAllGarages():
    url = "https://api.ibb.gov.tr/iett/UlasimAnaVeri/HatDurakGuzergah.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/GetGaraj_XML",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = """<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                  <Body>
                      <GetGaraj_XML xmlns="http://tempuri.org/"/>
                  </Body>
              </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()
    
    root = ET.fromstring(r.content)
    if tables := root.findall('*//Table'):
        results = []
        for table in tables:
            table = {GarageColumns[it.tag]: it.text for it in table.iter() if it.text}
            if coords := table.pop('KOORDINAT', None):
                pattern = r'POINT\s\((?P<Longitude>.*)\s(?P<Latitude>.*)\)'
                if matches := re.match(pattern, coords):
                    table.update()
            results.append(table)
        return results
    return None

def GetAnnouncements():
    url = "https://api.ibb.gov.tr/iett/UlasimDinamikVeri/Duyurular.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/GetDuyurular_XML",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = """<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                  <Body>
                      <GetDuyurular_XML xmlns="http://tempuri.org/"/>
                  </Body>
              </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()
    
    root = ET.fromstring(r.content)
    if tables := root.findall('*//Table'):
        results = []
        for table in tables:
            table = {AnnouncementColumns[it.tag]: it.text for it in table.iter() if it.text}
            if updateTime := table.get('UpdateTime'):
                pattern = r'Kayit Saati: \d{2}:\d{2}'
                if matches := re.match(pattern, updateTime):
                    updateTime = updateTime.replace('Kayit Saati: ', '')
                    table['UpdateTime'] = updateTime
            results.append(table)
        return results
    return None

def GetBadRoads():
    url = "https://api.ibb.gov.tr/iett/FiloDurum/SeferGerceklesme.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/GetBozukSatih_XML",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = """<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                  <Body>
                      <GetBozukSatih_XML xmlns="http://tempuri.org/"/>
                  </Body>
              </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()
    
    root = ET.fromstring(r.content)
    if tables := root.findall('*//Table'):
        results = []
        for table in tables:
            table = {BadRoadColums[it.tag]: it.text for it in table.iter() if it.text}
            table.update({
                'Longitude': float(table['Longitude']),
                'Latitude': float(table['Latitude'])
                })
            results.append(table)
        return results
    return None

def GetAccidentsByDate(dt):
    url = "https://api.ibb.gov.tr/iett/FiloDurum/SeferGerceklesme.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/GetKazaLokasyon_XML",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = f"""<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                   <Body>
                       <GetKazaLokasyon_XML xmlns="http://tempuri.org/">
                           <Tarih>{dt:%Y-%m-%d}</Tarih>
                       </GetKazaLokasyon_XML>
                   </Body>
               </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()
    
    root = ET.fromstring(r.content)
    if tables := root.findall('*//Table'):
        results = []
        for table in tables:
            table = {AccidentColums[it.tag]: it.text for it in table.iter() if it.text}
            table.update({
                'Longitude': float(table['Longitude']),
                'Latitude': float(table['Latitude'])
                })
            results.append(table)
        return results
    return None

def GetLineStopsByCode(lineCode):
    url = "https://api.ibb.gov.tr/iett/ibb/ibb.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/DurakDetay_GYY",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = f"""<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                   <Body>
                       <DurakDetay_GYY xmlns="http://tempuri.org/">
                           <hat_kodu>{lineCode}</hat_kodu>
                       </DurakDetay_GYY>
                   </Body>
               </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()
    
    root = ET.fromstring(r.content)
    if tables := root.findall('*//Table'):
        results = []
        for table in tables:
            table = {LineStopColums[it.tag]: it.text for it in table.iter() if it.text}
            table.update({
                'Longitude': float(table['Longitude']),
                'Latitude': float(table['Latitude'])
                })
            results.append(table)
        return results
    return None

def GetLineDetails(lineCode):
    url = "https://api.ibb.gov.tr/iett/ibb/ibb.asmx"
    headers = {
        "SOAPAction": "http://tempuri.org/HatServisi_GYY",
        "Content-type": "text/xml;charset=utf-8"
        }
    data = f"""<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                   <Body>
                       <HatServisi_GYY xmlns="http://tempuri.org/">
                           <hat_kodu>{lineCode}</hat_kodu>
                       </HatServisi_GYY>
                   </Body>
               </Envelope>"""
    r = requests.post(url, data=data, headers=headers)
    r.raise_for_status()

    root = ET.fromstring(r.content)
    if table := root.findall('*//Table'):
        table = {LineDetailColumns[it.tag]: it.text for it in table[0].iter() if it.text}
        return table
    return None










from datetime import datetime
import requests, re

def GetAQIStations() -> list[dict]:
    """Information of Air Quality stations is shared with this web service.

    Id (int): Contains StationId information. It is a unique field.
    
    Name (str): Contains the station name information.
    
    Address (str): Contains station address information.
    
    Latitude (float): Contains the Latitude of the station.
    
    Longitude (float): Contains the Longitude of the station.
    
    Returns:
        dict: Returns the list of stationId, Name, Address and Location.
    """
    with requests.Session() as s:
        url = 'https://api.ibb.gov.tr/havakalitesi/OpenDataPortalHandler/GetAQIStations'
        r = s.get(url, headers={'Accept-Encoding': 'gzip, deflate'})
        r.raise_for_status()

        pattern  = r'POINT\s\((?P<Lng>.*)\s(?P<Lat>.*)\)'
        stations = []

        for station in r.json():
            address  = station.pop('Adress')
            location = station.pop('Location')
            matches  = re.match(pattern, location)
            station.update({
                'Address': address,
                'Latitude': float(matches.group('Lat')),
                'Longitude': float(matches.group('Lng'))
                })
            stations.append(station)
            
        return stations
    
def GetAQIByStationId(stationId: str, startDate: datetime, endDate: datetime) -> tuple[list[dict], list[dict]]:
    """With this web service, Concentration and Air Quality Index (AQI)
    information is shared according to the start and end dates of the station
    whose ID is entered.
    
    Parameters:
        stationId (int): It is the Unique Id of the station whose measurement information will be retrieved.
        
        startDate (datetime): The specified date is determined as the startdate of the measurements.
        
        endDate (datetime): The specified date is determined as the end date of the measurements.
 
    Returns:
        list[dict]: PM10, SO2, O3, NO2 and CO measured from the Station's Analyzer contains the value of the data.
        
        list[dict]: The values of PM10, SO2, O3, NO2 and CO raw data measured from the Analyzer of the station after
        calculating the AirQuality Index and the HKI value for that hour, the Pollutant parameter,the status
        and the status includes color.
    """
    with requests.Session() as s:
        url = 'https://api.ibb.gov.tr/havakalitesi/OpenDataPortalHandler/GetAQIByStationId'
        params = {
            'StationId': stationId,
            'StartDate': startDate.strftime('%d.%m.%Y %H:%M:%S'),
            'EndDate': endDate.strftime('%d.%m.%Y %H:%M:%S')
            }
        r = s.get(url, params=params, headers={'Accept-Encoding': 'gzip, deflate'})
        r.raise_for_status()

        C, A = [], []
        for record in r.json():
            ReadTime = record.pop('ReadTime')
            ReadTime = datetime.fromisoformat(ReadTime).isoformat()
            if c := record.get('Concentration'):
                c |= {'ReadTime': ReadTime}
                C.append(c)
            if a := record.get('AQI'):
                a |= {'ReadTime': ReadTime}
                A.append(a)

        return C, A


import xml.etree.ElementTree as ET
import datetime, requests, enum

StationColumns = {
    'guid'       : 'Id',
    'istasyon_no': 'No',
    'adi'        : 'Name',
    'aktif'      : 'IsOpen',
    'bos'        : 'EmptyCapacity',
    'dolu'       : 'FullCapacity',
    'lat'        : 'Latitude',
    'lon'        : 'Longitude',
    'sonBaglanti': 'UpdateTime'
    }

def GetStationStatus(stationId: int) -> dict:
    """With this method (post), the status information of the stations
    serving on the İsbike application can be displayed.

    Parameters:
        stationId (int): Station guid (singular) information. (Available from the GetAllStationStatus method)
 
    Returns:
        Id (int): Unique ID of the station
        No (int): station number
        Name (str): station name
        IsOpen (bool): Station status information
        EmptyCapacity (int): Empty parking at the station information.
        FullCapacity (int): Park full at the station information.
        Latitude (float): The station's latitude information
        Longitude (float): Longitude of the station
        UpdateTime (str): Last access time to station
    """
    with requests.Session() as s:
        url = 'https://api.ibb.gov.tr/ispark-bike/GetStationStatus'
        headers = {'Accept-Encoding': 'gzip,deflate'}
        params = {'guid': stationId}

        r = requests.post(url, params=params, headers=headers)
        r.raise_for_status()

        station, = r.json().get('dataList')
        station = {newName: station[oldName] for oldName, newName in StationColumns.items()}
        station.update({
                    'No': int(station['No']),
                    'IsOpen': bool(station['IsOpen']),
                    'EmptyCapacity': int(station['EmptyCapacity']),
                    'FullCapacity': int(station['FullCapacity']),
                    'Latitude': float(station['Latitude']),
                    'Longitude': float(station['Longitude']),
                    'UpdateTime': station['UpdateTime'].rsplit('.', 1)[0]
                    })                
        return station
    
def GetAllStationStatus() -> list[dict]:
    """With this method (post), the status information of all stations
    serving on the İsbike application can be displayed.
 
    Returns:
        list[dict]: Returns list of all station information
    """
    with requests.Session() as s:
        url = 'https://api.ibb.gov.tr/ispark-bike/GetAllStationStatus'
        headers={'Accept-Encoding': 'gzip,deflate'}

        r = requests.post(url, headers=headers)
        r.raise_for_status()
        
        if stations := r.json().get('dataList'):
            _ = []
            for station in stations:
                station = {newName: station[oldName] for oldName, newName in StationColumns.items()}
                station.update({
                    'No': int(station['No']),
                    'IsOpen': bool(station['IsOpen']),
                    'EmptyCapacity': int(station['EmptyCapacity']),
                    'FullCapacity': int(station['FullCapacity']),
                    'Latitude': float(station['Latitude']),
                    'Longitude': float(station['Longitude']),
                    'UpdateTime': station['UpdateTime'].rsplit('.', 1)[0]
                    })
                _.append(station)
            stations = _
        return stations



import xml.etree.ElementTree as ET
import datetime, requests, enum

Month = enum.Enum('Month',('OCAK','ŞUBAT','MART','NİSAN','MAYIS','HAZIRAN','TEMMUZ','AĞUSTOS','EYLÜL','EKİM','KASIM','ARALIK'))

Landfills = {
	'Hekimbaşı Aktarma': '00000000000100000010',
	'Şile Aktarma' :'00000000000100000020',
	'Aydınlı Aktarma': '00000000000100000019',
	'Küçükbakkalköy Aktarma': '00000000000100000018',
	'Yenibosna Aktarma': '00000000000100000014',
	'Silivri Aktarma': '00000000000100000016',
	'Baruthane Aktarma': '00000000000100000021',
	'Halkalı Aktarma': '00000000000100000023',
	'Odayeri Düzenli Depolama': '00000000000100000012',
	'Geri Kazanım ve Kompost': '00000000000100000001',
	'Kömürcüoda Atık Bertaraf': '00000000000100000013',
	'Seymen Düzenli Depolama': '00000000000100000017',
	'Kemerburgaz IBB Atık Yakma ve Enerji Üretme': '00000000000100000162',
	'Odayeri Çöp Sızıntı Suyu Arıtma': '00000000000100000040',
	'Kömürcüoda Çöp Sızıntı Suyu Arıtma': '00000000000100000082'
	}

def GetTotalWeightByYear(stationId:str, year:int) -> list[dict]:
    """This method provides the amount of waste collected since 2016 in Transfer Stations.
    The input value must take one of the following:

    Station Id - Station Name
    
    [Asia Transfer Stations]
    00000000000100000010 – Hekimbaşı Aktarma
    00000000000100000020 – Şile Aktarma
    00000000000100000019 – Aydınlı Aktarma
    00000000000100000018 – Küçükbakkalköy Aktarma

    [Europa Transfer Stations]
    00000000000100000014 – Yenibosna Aktarma
    00000000000100000016 – Silivri Aktarma
    00000000000100000021 – Baruthane Aktarma
    00000000000100000023 – Halkalı Aktarma

    [Other Facilities]
    00000000000100000012 – Odayeri Düzenli Depolama
    00000000000100000001 – Geri Kazanım ve Kompost
    00000000000100000013 – Kömürcüoda Atık Bertaraf
    00000000000100000017 – Seymen Düzenli Depolama
    00000000000100000162 – Kemerburgaz IBB Atık Yakma ve Enerji Üretme
    00000000000100000040 – Odayeri Çöp Sızıntı Suyu Arıtma
    00000000000100000082 – Kömürcüoda Çöp Sızıntı Suyu Arıtma

    Parameters:
        stationId (str): The desired transfer stations id.
        year (int): The year for which the data is requested.
 
    Returns:
        list[dict]: District Municipality and montly total waste amounts of the entered year.
    """
    with requests.Session() as s:
        url = 'https://api.ibb.gov.tr/istac-zwr-toplam-yil-ws'
        headers={
            'Accept-Encoding':'gzip,deflate',
            'Content-Type': 'text/xml; charset=utf-8',
            'SOAPAction':'urn:sap-com:document:sap:rfc:functions:ZWR_TOPLAM_YIL_WS:ZWR_TOPLAM_TARTIM_FNRequest'
            }
        data = f"""<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                    <Body>
                     <ZWR_TOPLAM_TARTIM_FN xmlns="urn:sap-com:document:sap:rfc:functions">
                      <MJAHR xmlns="">{year}</MJAHR>
                      <ZWDPLANT xmlns="">{stationId}</ZWDPLANT>
                     </ZWR_TOPLAM_TARTIM_FN>
                    </Body>
                   </Envelope>"""

        r = requests.post(url, data=data, headers=headers)
        r.raise_for_status()

        root = ET.fromstring(r.content)
        records = []
        for it in root.findall('*//item'):
            year = it.find('MJAHR').text
            month = Month[it.find('AY').text].value  
            recordDate = f'{year}-{month:02d}-01'
            
            records.append({
                'RecordDate': recordDate,
                'City': it.find('CITY1').text,
                'Weight': float(it.find('EWEIGHT_NET').text)
                })
                
        return records


from datetime import datetime
import requests

ParkColumns = {
    'parkID'       : 'Id',
    'parkName'     : 'Name',
    'lat'          : 'Latitude',
    'lng'          : 'Longitude',
    'capacity'     : 'Capacity',
    'emptyCapacity': 'EmptyCapacity',
    'workHours'    : 'WorkHours',
    'parkType'     : 'Type',
    'freeTime'     : 'FreeTime',
    'district'     : 'District',
    'isOpen'       : 'IsOpen'
    }

DetailColumns = {
    'parkID'       : 'Id',
    'parkName'     : 'Name',
    'lat'          : 'Latitude',
    'lng'          : 'Longitude',
    'capacity'     : 'Capacity',
    'emptyCapacity': 'EmptyCapacity',
    'workHours'    : 'WorkHours',
    'parkType'     : 'Type',
    'freeTime'     : 'FreeTime',
    'district'     : 'District',
    'locationName' : 'LocationName',
    'updateDate'   : 'UpdateTime',
    'monthlyFee'   : 'MonthlyFee',
    'tariff'       : 'Tariff',
    'address'      : 'Address',
    'areaPolygon'  : 'AreaPolygon'
    }

def GetAllParks() -> list[dict]:
    """This method will return the information of all Ispark car parks.
    The returning list will have dictionary with the following parameters

    Capacity (int): The total capacity of the park.
    
    District (str): District where the park is located.
    
    EmptyCapacity (int): Empty capacity of the park.
    
    FreeTime (int): Free parking time (minute).
    
    Id (int): Contains Park Id information. It is a unique field.
    
    IsOpen (bool): Is the park open.
    
    Latitude (float): Latitude information of the park.
    
    Longitude (float): Longitude information of the park.
    
    Name (str): Contains park name information.
    
    Type (str): Type of park (on road, open, closed).
    
    WorkHours (str): Working hours of the park.
    
    Returns:
        list[dict]: The information of all Ispark car parks
    """
    with requests.Session() as s:        
        url = 'https://api.ibb.gov.tr/ispark/Park'
        r = s.get(url, headers={'Accept-Encoding': 'gzip,deflate'})
        r.raise_for_status()

        parks = []
        for park in r.json():
            park = {newName: park[oldName] for oldName, newName in ParkColumns.items()}
            park.update({
                'Latitude': float(park['Latitude']),
                'Longitude': float(park['Longitude']),
                'IsOpen': park['IsOpen'] != 0
                })
            parks.append(park)
            
        return parks
    
def GetParkDetail(parkId: int) -> dict:
    """With this web service, detailed information of any Ispark car park
    can be accessed with given parkId

    Id (int): Contains Park Id information. It is a unique field.
    
    Name (str): Contains park name information.
    
    Latitude (float): Contains park latitude information.
    
    Longitude (float): Contains park longitude information.
    
    Capacity (int): The total capacity of the park.
    
    EmptyCapacity (int): The empty capacity of the park.
    
    WorkHours (str): Park opening hours.
    
    Type (str): Type of park (on road, open, closed).
    
    FreeTime (int): Free parking time (minute).
    
    District (str): District where the park is located.
    
    LocationName (str): Contains location name information.
    
    UpdateTime (str): Update date and time of information
    
    MonthlyFee (float): Monthly Subscription Fee.
    
    Tariff (dict[str, float]): List of Tariff Information object with Tariff(string) and Price(float) key-value pairs
    
    Address (str): Parking lot address.
    
    AreaPolygon (str): Parking polygon information.
    
    Parameters:
        parkId (int): The desired park id.
 
    Returns:
        dict: Detailed information about desired car park.
    """
    with requests.Session() as s:        
        url = 'https://api.ibb.gov.tr/ispark/ParkDetay'
        details = []

        r = s.get(url, params={'id': parkId}, headers={'Accept-Encoding': 'gzip,deflate'})
        r.raise_for_status()

        detail, = r.json()
        detail = {newName: detail[oldName] for oldName, newName in DetailColumns.items()}

        tariff = {}
        for t in detail['Tariff'].split(';'):
            interval, price = t.replace(',', '.').split(':')
            tariff[interval.rstrip()] = float(price)

        if updateTime := detail['UpdateTime']:
            updateTime = datetime.strptime(updateTime, '%d.%m.%Y %H:%M:%S').isoformat()

        detail.update({
            'Tariff': tariff,
            'Latitude': float(detail['Latitude']),
            'Longitude': float(detail['Longitude']),
            'UpdateTime': updateTime
            })
        
        return detail